@@ -13,6 +13,7 @@ import os.path
 from pprint import pprint
 import re
 import sys
+import time
 import warnings
 try:                    # Python 3
     import configparser
@@ -59,10 +60,11 @@ except ImportError:
         return _convert(node_or_string)
 
 
-__version__ = '0.11'
+__version__ = '1.0'
 __all__ = ['Client', 'Model', 'Record', 'RecordList', 'Service', 'read_config']
 
 CONF_FILE = 'erppeek.ini'
+HIST_FILE = os.path.expanduser('~/.erppeek_history')
 DEFAULT_URL = 'http://localhost:8069'
 DEFAULT_DB = 'openerp'
 DEFAULT_USER = 'admin'
@@ -117,7 +119,8 @@ _fields_re = re.compile(r'(?:[^%]|^)%\(([^)]+)\)')
 
 # Published object methods
 _methods = {
-    'db': ['create', 'drop', 'dump', 'restore', 'rename', 'list', 'list_lang',
+    'db': ['create', 'drop', 'dump', 'restore', 'rename',
+           'get_progress', 'list', 'list_lang',
            'change_admin_password', 'server_version', 'migrate_databases'],
     'common': ['about', 'login', 'timezone_get', 'get_server_environment',
                'login_message', 'check_connectivity'],
@@ -136,8 +139,8 @@ _methods_6_1 = {
 # Hidden methods:
 #  - (not in 6.1) 'common': ['logout', 'ir_get', 'ir_set', 'ir_del']
 #  - (not in 6.1) 'object': ['obj_list']
-#  - 'db': ['get_progress']
-#  - 'common': ['get_available_updates', 'get_migration_scripts', 'set_loglevel']
+#  - 'common': ['get_available_updates', 'get_migration_scripts',
+#               'set_loglevel']
 
 
 def mixedcase(s, _cache={}):
@@ -310,13 +313,15 @@ class Client(object):
     """
     _config_file = os.path.join(os.path.curdir, CONF_FILE)
 
-    def __init__(self, server, db, user, password=None, verbose=False):
+    def __init__(self, server, db=None, user=None, password=None,
+                 verbose=False):
         self._server = server
-        self._db = db
+        self._db = ()
         self._environment = None
         self.user = None
-        major_version = None
         self._execute = None
+        self._models = {}
+        major_version = None
 
         def get_proxy(name):
             if major_version in ('5.0', None):
@@ -333,9 +338,9 @@ class Client(object):
         self._object = get_proxy('object')
         self._wizard = get_proxy('wizard')
         self._report = get_proxy('report')
-        # Try to login
-        self._login(user, password)
-        self._models = {}
+        if db:
+            # Try to login
+            self._login(user, password=password, database=db)
 
     @classmethod
     def from_config(cls, environment, verbose=False):
@@ -353,16 +358,31 @@ class Client(object):
     def __repr__(self):
         return "<Client '%s#%s'>" % (self._server, self._db)
 
-    def login(self, user, password=None):
-        """Switch `user`.
+    def login(self, user, password=None, database=None):
+        """Switch `user` and (optionally) `database`.
 
-        If the `password` is not provided, it will be asked.
+        If the `password` is not available, it will be asked.
         """
+        if database:
+            dbs = self.db.list()
+            if database not in dbs:
+                print("Error: Database '%s' does not exist: %s" %
+                      (database, dbs))
+                return
+            if not self._db:
+                self._db = database
+        elif not self._db:
+            print('Error: Not connected')
+            return
         (uid, password) = self._auth(user, password)
         if uid is False:
             print('Error: Invalid username or password')
             return
         self.user = user
+        if database:
+            if self._db != database:
+                self._environment = None
+            self._db = database
 
         # Authenticated endpoints
         def authenticated(method):
@@ -393,6 +413,7 @@ class Client(object):
             return False
 
     def _auth(self, user, password):
+        assert self._db
         cache_key = (self._server, self._db, user)
         if password:
             # If password is explicit, call the 'login' method
@@ -402,7 +423,8 @@ class Client(object):
             uid, password = self._login.cache.get(cache_key) or (None, None)
             # Read from table 'res.users'
             if not uid and self.access('res.users', 'write'):
-                obj = self.read('res.users', [('login', '=', user)], 'id password')
+                obj = self.read('res.users',
+                                [('login', '=', user)], 'id password')
                 if obj:
                     uid = obj[0]['id']
                     password = obj[0]['password']
@@ -435,6 +457,7 @@ class Client(object):
                         'model', 'models', 'keys', 'fields', 'field', 'access']
 
         def connect(self, env=None):
+            """Connect to another environment and replace the globals()."""
             if env:
                 client = self.from_config(env, verbose=self.db._verbose)
             else:
@@ -443,8 +466,8 @@ class Client(object):
             g = globals()
             g['client'] = client
             # Tweak prompt
-            sys.ps1 = '%s >>> ' % env
-            sys.ps2 = '%s ... ' % env
+            sys.ps1 = '%s >>> ' % (env,)
+            sys.ps2 = '%s ... ' % (env,)
             # Logged in?
             if client.user:
                 g['do'] = client.execute
@@ -455,15 +478,39 @@ class Client(object):
                 g['do'] = None
                 g.update(dict.fromkeys(global_names))
 
-        def login(self, user):
-            if self._login(user):
-                # If successful, register the new globals()
+        def login(self, user, database=None):
+            """Switch `user` and (optionally) `database`."""
+            if self._login(user, database=database):
+                # Register the new globals()
                 self.connect()
 
         # Set hooks to recreate the globals()
         cls.login = login
         cls.connect = connect
 
+    def create_database(self, passwd, database, demo=False, lang='en_US',
+                        user_password='admin'):
+        """Create a new database.
+
+        The superadmin `passwd` and the `database` name are mandatory.
+        By default, `demo` data are not loaded and `lang` is ``en_US``.
+        Wait for the thread to finish and login if successful.
+        """
+        thread_id = self.db.create(passwd, database, demo, lang, user_password)
+        progress = 0
+        try:
+            while progress < 1:
+                time.sleep(3)
+                progress, users = self.db.get_progress(passwd, thread_id)
+            # [1.0, [{'login': 'admin', 'password': 'admin',
+            #         'name': 'Administrator'}]]
+            self._login(users[0]['login'], users[0]['password'],
+                        database=database)
+            if self.connect is not None:
+                self.connect()
+        except KeyboardInterrupt:
+            print({'id': thread_id, 'progress': progress})
+
     def execute(self, obj, method, *params, **kwargs):
         """Wrapper around ``object.execute`` RPC method.
 
@@ -740,7 +787,8 @@ class Client(object):
             self.__dict__[method] = rv
             return rv
         if method.startswith('__'):
-            raise AttributeError("'Client' object has no attribute %r" % method)
+            raise AttributeError("'Client' object has no attribute %r" %
+                                 method)
 
         # miscellaneous object methods
         def wrapper(self, obj, *params, **kwargs):
@@ -822,6 +870,9 @@ class Model(object):
         if issearchdomain(domain):
             params = searchargs((domain,) + params, kwargs, context)
             ids = self._execute('search', *params)
+            # Ignore extra keyword arguments
+            for item in kwargs.items():
+                print('Ignoring: %s = %r' % item)
         return RecordList(self, ids, context=context)
 
     # alias
@@ -883,13 +934,18 @@ class RecordList(object):
     """A sequence of OpenERP :class:`Record`."""
 
     def __init__(self, res_model, ids, context=None):
-        self._model = res_model
-        self._idnames = ids
         if ids and isinstance(ids[0], list):
-            self._ids = [id_ for (id_, name) in ids]
+            _ids = [id_ for (id_, name) in ids]
         else:
-            self._ids = ids
-        self._context = context
+            _ids = ids
+        # Bypass the __setattr__ method
+        self.__dict__.update({
+            # 'client': res_model.client,
+            '_model': res_model,
+            '_idnames': ids,
+            '_ids': _ids,
+            '_context': context,
+        })
 
     def __repr__(self):
         if len(self._ids) > 16:
@@ -902,6 +958,9 @@ class RecordList(object):
         return ['__getitem__', 'read', 'write', 'unlink',
                 '_context', '_ids', '_idnames', '_model']
 
+    def __len__(self):
+        return len(self._ids)
+
     def read(self, fields=None, context=None):
         """Wrapper for :meth:`Record.read` method."""
         if context is None and self._context:
@@ -952,6 +1011,13 @@ class RecordList(object):
         self.__dict__[attr] = mobj = wrapper.__get__(self, type(self))
         return mobj
 
+    def __setattr__(self, attr, value):
+        if attr in self._model._keys:
+            msg = "attribute %r is read-only"
+        else:
+            msg = "has no attribute %r"
+        raise AttributeError("'RecordList' object %s" % msg % attr)
+
 
 class Record(object):
     """A class for all OpenERP records.
@@ -1045,6 +1111,9 @@ class Record(object):
         self._clear_cache()
         return rv
 
+    # alias
+    update = write
+
     def unlink(self, context=None):
         """Delete the current :class:`Record` from the database."""
         if context is None and self._context:
@@ -1093,13 +1162,13 @@ class Record(object):
         return mobj
 
     def __setattr__(self, attr, value):
-        if attr in self._model._keys:
-            assert attr != 'id'
-            self.write({attr: value})
-            if attr in self.__dict__:
-                delattr(self, attr)
-        else:
-            object.__setattr__(self, attr, value)
+        if attr not in self._model._keys:
+            raise AttributeError("'Record' object has no attribute %r" % attr)
+        if attr == 'id':
+            raise AttributeError("'Record' object attribute 'id' is read-only")
+        self.write({attr: value})
+        if attr in self.__dict__:
+            delattr(self, attr)
 
 
 def _interact(use_pprint=True, usage=USAGE):
@@ -1143,9 +1212,19 @@ def _interact(use_pprint=True, usage=USAGE):
     builtins.usage = Usage()
 
     try:
-        __import__('readline')
+        import readline as rl
+        import rlcompleter
     except ImportError:
         pass
+    else:
+        import atexit
+        rl.parse_and_bind('tab: complete')
+        if os.path.exists(HIST_FILE):
+            rl.read_history_file(HIST_FILE)
+            if rl.get_history_length() < 0:
+                rl.set_history_length(int(os.environ.get('HISTSIZE', 500)))
+            # better append instead of replace?
+            atexit.register(rl.write_history_file, HIST_FILE)
 
     class Console(code.InteractiveConsole):
         def runcode(self, code):
@@ -1167,7 +1246,8 @@ def main():
                    'or query a model (-m) and pass search terms or '
                    'ids as positional parameters after the options.')
     parser = optparse.OptionParser(
-        usage='%prog [options] [search_term_or_id [search_term_or_id ...]]', version=__version__,
+        usage='%prog [options] [search_term_or_id [search_term_or_id ...]]',
+        version=__version__,
         description=description)
     parser.add_option(
         '-l', '--list', action='store_true', dest='list_env',
